#-----------------------------------------------------------------------
# Modified 2003 Boris Schfer <menc@noid-project>
# for the Video::Menc project
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This module is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::Content;

use base Video::Menc::Base;

use Video::Menc::ContentProbe;
use Video::Menc::Title;

use Carp;
use strict;

sub project			{ shift->{project}		}
sub titles			{ shift->{titles}  		} # href/undef
sub probe_result	{ shift->{probe_result}	}

sub set_titles			{ shift->{titles}		= $_[1] }
sub set_probe_result	{ shift->{probe_result}	= $_[1] }

sub set_project	{
	my $self = shift;
	my ($project) = @_;

	$self->{project} = $project;
	
	return if not $self->titles;

	foreach my $title ( values %{$self->titles} ) {
		$title->set_project ($project);
	}
	
	return $project;
}

sub new {
	my $class = shift;
	my %par = @_;
	my  ($project) = @par{'project'};

	my $self = {
		project 		=> $project,
		titles  		=> undef,
		probe_result 	=> undef,
	};
	
	return bless $self, $class;
}

sub read_title_listing {
	my $self = shift;

	my $titlecount = $self->probe_result->titles;
	my $title;
	my %titles;
	
	my $project = $self->project;
	my $dvd_device = $project->dvd_device;
	
	for (my $i=1; $i <= $titlecount; $i++) {
  		$title = $titles{$i};
		$title ||= Video::Menc::Title->new (
			dvdtrack => $i,
			project => $project
		);
		$titles{$i} ||= $title;
		
		$title->set_type("dvd");
		$title->set_postcmd(" -dvd $i -dvd-device $dvd_device ");
	}	

	# story Title objects
	$self->set_titles (\%titles);

 	1;
}

sub get_titles {
	my $self = shift;
	
	$self->read_title_listing if not $self->titles;
	
	my @titles = sort { $a->dvdtrack <=> $b->dvdtrack } values %{$self->titles};
	
	return \@titles;
}

#---------------------------------------------------------------------
# Methods for Probing DVD
#---------------------------------------------------------------------

sub get_dvd_content_command {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($dvdtrack, $audioid, $notcprobe) = 
		@par{'dvdtrack', 'audioid', 'notcprobe'};
		
	my $dvd_device    = $self->project->dvd_device;
	my $audio = "-aid $audioid" if ($audioid);
    my $tcprobe = "tcprobe -i $dvd_device -T $dvdtrack";
	$tcprobe = "" if ($notcprobe);
	
#print "CMD = 'mplayer -identify -v -dvd-device $dvd_device ".
#	       "-dvd $dvdtrack $audio ; $tcprobe'\n";	
	return "mplayer -identify -v -dvd-device $dvd_device ".
	       "-dvd $dvdtrack $audio ; $tcprobe";
}

sub dvd_content_async_start {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($dvdtrack, $audioid, $notcprobe) = 
		@par{'dvdtrack', 'audioid', 'notcprobe'};
				
	return $self->popen (
		command => $self->get_dvd_content_command (
						dvdtrack  => $dvdtrack,
						audioid   => $audioid,
						notcprobe => $notcprobe,
				   ),
	);
}

sub dvd_content_async_stop {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($fh, $output, $dvdtrack, $audioid, $notcprobe) = 
		@par{'fh','output', 'dvdtrack', 'audioid', 'notcprobe'};

	$output = "\n\nOutput was:\n\n$output" if $output;

	my $message =   "Error executing:\n\n".
			$self->get_dvd_content_command (
				dvdtrack  => $dvdtrack,
				audioid   => $audioid,
				notcprobe => $notcprobe,
			).
			$output;

	close $fh or croak ($message);
	
	$self->analyze_dvd_content_output (
		output => $output
	);
	
	1;
}

sub analyze_dvd_content_output {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($output) = @par{'output'};

	$self->set_probe_result (
		Video::Menc::ContentProbe->analyze_dvd (
			probe_output => $output,
		)
	);
	
	1;
}

#---------------------------------------------------------------------
# Methods for Probing File
#---------------------------------------------------------------------

sub get_file_content_command {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($filename, $audioid) = 
		@par{'filename', 'audioid'};
		
	my $audio = "-aid $audioid" if ($audioid);

	return "mplayer -identify -v $audio \"$filename\" ";
}

sub file_content_async_start {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($filename, $audioid) = 
		@par{'filename', 'audioid'};
				
	return $self->popen (
		command => $self->get_file_content_command (
						filename  => $filename,
						audioid   => $audioid,
				   ),
	);
}

sub file_content_async_stop {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($fh, $output, $filename, $audioid) = 
		@par{'fh','output', 'filename', 'audioid'};

	$output = "\n\nOutput was:\n\n$output" if $output;

	my $message =   "Error executing:\n\n".
			$self->get_dvd_content_command (
				filename  => $filename,
				audioid   => $audioid,
			).
			$output;

	close $fh or croak ($message);
	
	$self->analyze_file_content_output (
		output => $output
	);
	
	1;
}

sub analyze_file_content_output {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($output) = @par{'output'};

	$self->set_probe_result (
		Video::Menc::ContentProbe->analyze_file (
			probe_output => $output,
		)
	);
	
	1;
}

#---------------------------------------------------------------------
# Methods for Probing DBox2 File
#---------------------------------------------------------------------

sub get_dbox2_content_command {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($filename, $step, $seek, $audioid) = 
		@par{'filename', 'step', 'seek', 'audioid'};

    if ( $audioid ne "" ) {
	    return "mplayer -v -identify -aid $audioid \"$filename\"";
	}
	if ( $step == 0) {
		return "mplayer -identify \"$filename\" ; mplayer -nosound -vo null -frames 2 \"$filename\" ";
	} else {
		return "mplayer -nosound -vo null -ss $seek \"$filename\" "; 
	}		
}

sub dbox2_content_async_start {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($filename, $step, $seek, $audioid) = 
		@par{'filename', 'step', 'seek', 'audioid'};
				
	return $self->popen (
		command => $self->get_dbox2_content_command (
						filename => $filename,
						step     => $step,
						seek     => $seek,
						audioid  => $audioid,
				   ),
	);
}

sub dbox2_content_async_stop {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($fh, $output, $filename, $step, $seek, $audioid) = 
		@par{'fh','output', 'filename', 'step', 'seek', 'audioid'};

	$output = "\n\nOutput was:\n\n$output" if $output;

	my $message =   "Error executing:\n\n".
			$self->get_dbox2_content_command (
				filename => $filename,
				step     => $step,
				seek     => $seek,
				audioid  => $audioid,
			).
			$output;

	close $fh or croak ($message);
	
	$self->analyze_dbox2_content_output (
		output  => $output,
		step    => $step,
		audioid => $audioid,
	);
	
	1;
}

sub analyze_dbox2_content_output {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($output, $step, $audioid) = @par{'output', 'step', 'audioid'};

	$self->set_probe_result (
		Video::Menc::ContentProbe->analyze_dbox2 (
			probe_output => $output,
			step         => $step,
			audioid      => $audioid,
		)
	);
	
	1;
}

#---------------------------------------------------------------------
# Methods for Probing DBox2 A/V Sync
#---------------------------------------------------------------------

sub get_dbox2_avsync_command {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($filename) = @par{'filename'};

	return "mencoder -oac mp3lame -lameopts cbr:br=128:q=0 ".
	       "-ovc lavc -lavcopts vcodec=mpeg4:vbitrate=2000:vhq:keyint=250 ".
		   "-o /dev/null -mc 10 -endpos 2 \"$filename\" ";
}

sub dbox2_avsync_async_start {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($filename) = @par{'filename'};
				
	return $self->popen (
		command => $self->get_dbox2_avsync_command (
						filename => $filename,
				   ),
	);
}

sub dbox2_avsync_async_stop {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($fh, $output, $filename) = 
		@par{'fh','output', 'filename'};

	$output = "\n\nOutput was:\n\n$output" if $output;

	my $message =   "Error executing:\n\n".
			$self->get_dbox2_avsync_command (
				filename => $filename,
			).
			$output;

	close $fh or croak ($message);
	
	$self->analyze_dbox2_avsync_output (
		output => $output,
	);
	
	1;
}

sub analyze_dbox2_avsync_output {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($output) = @par{'output'};

	$self->set_probe_result (
		Video::Menc::ContentProbe->analyze_dbox2_avsync (
			probe_output => $output,
		)
	);
	
	1;
}

1;
