#-----------------------------------------------------------------------
# Modified 2003 Boris Schfer <menc@noid-project>
# for the Video::Menc project
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This module is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::Project;

use base Video::Menc::Base;

use Carp;
use strict;

use FileHandle;
use Data::Dumper;

use Video::Menc::Content;

sub version				{ shift->{version}				}
sub avifile				{ shift->{avifile}				}
sub dvd_device			{ shift->{dvd_device}			}
sub content				{ shift->{content}				}
sub last_saved_data		{ shift->{last_saved_data}		}
sub selected_title_nr	{ shift->{selected_title_nr}	}

sub oac						{ shift->{oac}						}
sub oac2					{ shift->{oac2}						}
sub twopass					{ shift->{twopass}					}
sub vop_lavcdeint			{ shift->{vop_lavcdeint}			}
sub vop_field				{ shift->{vop_field}				}
sub vop_avsync				{ shift->{vop_avsync}				}
sub use_bpp					{ shift->{use_bpp}					}
sub target_bpp				{ shift->{target_bpp}				}
sub bpp_popup				{ shift->{bpp_popup}				}
sub disc_count				{ shift->{disc_count}				}
sub disc_size				{ shift->{disc_size}				}
sub target_size				{ shift->{target_size}				}
sub video_length			{ shift->{video_length}				}
sub video_bitrate			{ shift->{video_bitrate}			}
sub audio_bitrate			{ shift->{audio_bitrate}			}
sub audio2_bitrate			{ shift->{audio2_bitrate}			}
sub target_bpp_default		{ shift->{target_bpp_default}		}
sub target_size_default		{ shift->{target_size_default}		}
sub video_length_default	{ shift->{video_length_default}		}
sub video_bitrate_default	{ shift->{video_bitrate_default}	}
sub audio_bitrate_default	{ shift->{audio_bitrate_default}	}
sub audio2_bitrate_default	{ shift->{audio2_bitrate_default}	}
sub cut_start				{ shift->{cut_start}				}
sub cut_end					{ shift->{cut_end}					}

sub set_version				{ shift->{version}				= $_[1] }
sub set_avifile				{ shift->{avifile}				= $_[1] }
sub set_dvd_device			{ shift->{dvd_device}			= $_[1]	}
sub set_content				{ shift->{content}				= $_[1] }
sub set_last_saved_data		{ shift->{last_saved_data}		= $_[1] }
sub set_selected_title_nr	{ shift->{selected_title_nr}	= $_[1] }

sub set_oac						{ shift->{oac}						= $_[1] }
sub set_oac2					{ shift->{oac2}						= $_[1] }
sub set_twopass					{ shift->{twopass}					= $_[1] }
sub set_vop_lavcdeint			{ shift->{vop_lavcdeint}			= $_[1] }
sub set_vop_field				{ shift->{vop_field}				= $_[1] }
sub set_vop_avsync				{ shift->{vop_avsync}				= $_[1] }
sub set_use_bpp					{ shift->{use_bpp}					= $_[1] }
sub set_target_bpp				{ shift->{target_bpp}				= $_[1] }
sub set_bpp_popup				{ shift->{bpp_popup}				= $_[1] }
sub set_disc_count				{ shift->{disc_count}				= $_[1] }
sub set_disc_size				{ shift->{disc_size}				= $_[1] }
sub set_target_size				{ shift->{target_size}				= $_[1] }
sub set_video_length			{ shift->{video_length}				= $_[1] }
sub set_video_bitrate			{ shift->{video_bitrate}			= $_[1]	}
sub set_audio_bitrate			{ shift->{audio_bitrate}			= $_[1]	}
sub set_audio2_bitrate			{ shift->{audio2_bitrate}			= $_[1]	}
sub set_target_bpp_default		{ shift->{target_bpp_default}		= $_[1]	}
sub set_target_size_default		{ shift->{target_size_default}		= $_[1]	}
sub set_video_length_default	{ shift->{video_length_default}		= $_[1]	}
sub set_video_bitrate_default	{ shift->{video_bitrate_default}	= $_[1]	}
sub set_audio_bitrate_default	{ shift->{audio_bitrate_default}	= $_[1]	}
sub set_audio2_bitrate_default	{ shift->{audio2_bitrate_default}	= $_[1]	}
sub set_cut_start				{ shift->{cut_start}				= $_[1]	}
sub set_cut_end					{ shift->{cut_end}					= $_[1]	}

sub new {
	my $class = shift;
	my %par = @_;
	
	my $self = {
		version           		=> $Video::Menc::VERSION,
		avifile          		=> undef,
		dvd_device	  			=> "/dev/dvd",
		content  	  			=> undef,
		last_saved_data   		=> undef,
		selected_title_nr 		=> undef,
		oac						=> 1,
		oac2					=> 1,
		twopass					=> 1,
		vop_lavcdeint			=> 0,
		vop_field				=> -1,
		vop_avsync				=> undef,
		use_bpp					=> 1,
		disc_count				=> 2,
		disc_size				=> 700,
		target_size				=> undef,
		video_length			=> undef,
		video_bitrate			=> undef,
		audio_bitrate			=> undef,
		audio2_bitrate			=> undef,
		target_bpp_default		=> 0.2,
		target_size_default		=> 1400,
		video_length_default	=> undef,
		video_bitrate_default	=> undef,
		audio_bitrate_default	=> undef,
		audio2_bitrate_default	=> undef,
		cut_start				=> undef,
		cut_end					=> undef,
	};
	
	bless $self, $class;

	my $content = Video::Menc::Content->new ( project => $self );

	$self->set_content ($content);

	return $self;
}

sub new_from_file {
	my $class = shift;
	my %par = @_;
	my  ($filename) =
	@par{'filename'};
	
	confess "missing filename" if not $filename;
	
	my $self = bless {
		filename => $filename,
	}, $class;
	
	$self->load;
	
	$self->set_filename ($filename);
	$self->set_version ($Video::Menc::VERSION);

	return $self;
}

sub save {
	my $self = shift; $self->trace_in;
	
	my $filename = $self->filename;
	confess "not filename set" if not $filename;
	
	my $data_sref = $self->get_save_data;
	
	my $fh = FileHandle->new;

	open ($fh, "> $filename") or confess "can't write $filename";
	print $fh q{# $Id: Project.pm,v 1.15 2001/11/29 20:38:03 joern Exp $},"\n";
	print $fh "# This file was generated by Video::Menc Version $Video::Menc::VERSION\n\n";

	print $fh ${$data_sref};
	close $fh;
	
	$self->set_last_saved_data ($data_sref);

	1;
}

sub get_save_data {
	my $self = shift; $self->trace_in;
	
	my $last_saved_data = $self->last_saved_data;
	$self->set_last_saved_data(undef);

	my $filename = $self->filename;
	$self->set_filename(undef);

	my $dd = Data::Dumper->new ( [$self], ['project'] );
	$dd->Indent(1);
	my $data = $dd->Dump;

	$self->set_last_saved_data($last_saved_data);
	$self->set_filename ($filename);

	return \$data;
}

sub changed {
	my $self = shift; $self->trace_in;

	return 1 if not $self->last_saved_data;

	my $actual_data_sref = $self->get_save_data;
	my $saved_data_sref  = $self->last_saved_data;
	
	my $actual = join ("\n", sort split (/\n/, $$actual_data_sref));
	my $saved  = join ("\n", sort split (/\n/, $$saved_data_sref));
	
	return $actual ne $saved;
}

sub load {
	my $self = shift; $self->trace_in;
	
	my $filename = $self->filename;
	croak "no filename set" if not $filename;
	croak "can't read $filename" if not -r $filename;
	
	my $fh = FileHandle->new;
	open ($fh, $filename) or croak "can't read $filename";
	my $data = join ('', <$fh>);
	close $fh;

	croak "File is no menc file"
		if $data !~ /This file was generated by Video::Menc/;

	my $project;
	$project = eval($data);
	croak "can't load $filename. Perl error: $@" if $@;

	%{$self} = %{$project};

	$self->content->set_project ($self);

	$self->set_last_saved_data ($self->get_save_data);
	
	1;
}

1;
