#-----------------------------------------------------------------------
# Modified 2003 Boris Schfer <menc@noid-project>
# for the Video::Menc project
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Copyright (C) 2001-2002 Jrn Reder <joern@zyn.de> All Rights Reserved
# 
# This module is part of Video::DVDRip, which is free software; you can
# redistribute it and/or modify it under the same terms as Perl itself.
#-----------------------------------------------------------------------

package Video::Menc::Title;

use base Video::Menc::Base;

use Video::Menc::Probe;

use Carp;
use strict;

use File::Path;
use File::Basename;
use File::Copy;

#------------------------------------------------------------------------
# These attributes are probed from the DVD
#------------------------------------------------------------------------

sub type			{ shift->{type}			}
sub precmd			{ shift->{precmd}		}
sub postcmd         { shift->{postcmd}		}
sub audiocmd		{ shift->{audiocmd}		}
sub audiocmd2		{ shift->{audiocmd2}	}

sub dvdtrack		{ shift->{dvdtrack}		}
sub filename		{ shift->{filename}		}

sub width			{ shift->{width}		}
sub height			{ shift->{height}		}
sub aspect			{ shift->{aspect}		}
sub runtime			{ shift->{runtime}		}
sub dboxruntime		{ shift->{dboxruntime}	}
sub dboxavsync		{ shift->{dboxavsync}	}
sub fps				{ shift->{fps}			}
sub audio_tracks	{ shift->{audio_tracks}	}

sub adjust_width	{ shift->{adjust_width}		}
sub adjust_height	{ shift->{adjust_height}	}

sub audio_channel	{ shift->{audio_channel}	}
sub audio2_channel	{ shift->{audio2_channel}	}
sub preset			{ shift->{preset}			}
sub preview_second	{ shift->{preview_second}	}
sub files			{ shift->{files}			}

sub set_type			{ shift->{type}			= $_[1]	}
sub set_precmd			{ shift->{precmd}		= $_[1]	}
sub set_postcmd			{ shift->{postcmd}		= $_[1]	}
sub set_audiocmd		{ shift->{audiocmd}		= $_[1]	}
sub set_audiocmd2		{ shift->{audiocmd2}	= $_[1]	}

sub set_dvdtrack		{ shift->{dvdtrack}		= $_[1] }
sub set_filename		{ shift->{filename}		= $_[1] }

sub set_width			{ shift->{width}		= $_[1]	}
sub set_height			{ shift->{height}		= $_[1]	}
sub set_aspect			{ shift->{aspect}		= $_[1]	}
sub set_runtime			{ shift->{runtime}		= $_[1]	}
sub set_dboxruntime		{ shift->{dboxruntime}	= $_[1] }
sub set_dboxavsync		{ shift->{dboxavsync}	= $_[1] }
sub set_fps				{ shift->{fps}			= $_[1]	}
sub set_audio_tracks	{ shift->{audio_tracks}	= $_[1]	}

sub set_adjust_width	{ shift->{adjust_width}		= $_[1] }
sub set_adjust_height	{ shift->{adjust_height}	= $_[1] }

sub set_audio_channel	{ shift->{audio_channel}	= $_[1] }
sub set_audio2_channel	{ shift->{audio2_channel}	= $_[1] }
sub set_probe_result	{ shift->{probe_result}		= $_[1] }
sub set_preset			{ shift->{preset}			= $_[1] }
sub set_preview_second	{ shift->{preview_second}	= $_[1] }

#------------------------------------------------------------------------
# These attributes must be specified by the user and are
# input parameters for the transcode process.
#------------------------------------------------------------------------

sub project				{ shift->{project}				}
sub tc_crop_top			{ shift->{tc_crop_top}			}
sub tc_crop_bottom		{ shift->{tc_crop_bottom}		}
sub tc_crop_left		{ shift->{tc_crop_left}			}
sub tc_crop_right		{ shift->{tc_crop_right}		}
sub tc_scale_width		{ shift->{tc_scale_width}		}
sub tc_scale_height		{ shift->{tc_scale_height}		}

sub set_project				{ shift->{project}				= $_[1] }
sub set_tc_crop_top			{ shift->{tc_crop_top}			= $_[1]	}
sub set_tc_crop_bottom		{ shift->{tc_crop_bottom}		= $_[1]	}
sub set_tc_crop_left		{ shift->{tc_crop_left}			= $_[1]	}
sub set_tc_crop_right		{ shift->{tc_crop_right}		= $_[1]	}
sub set_tc_scale_width		{ shift->{tc_scale_width}		= $_[1]	}
sub set_tc_scale_height		{ shift->{tc_scale_height}		= $_[1]	}

sub preview_filename {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($type) = @par{'type'};

    if ( $self->type eq undef ) {
		return "/tmp/menc$$/preview-$type.jpg";		
	}	
	if ( $self->type eq "dvd" ) {
		return 	"/tmp/menc$$/".
			$self->dvdtrack."-preview-$type.jpg";
	}
	if ( $self->type eq "file" ) {
		return 	"/tmp/menc$$/".
			"file-preview-$type.jpg";
	}	
}

sub new {
	my $class = shift;
	my %par = @_;
	my ($dvdtrack, $filename, $project) = 
		@par{'dvdtrack','filename','project'};

	my $self = {
		type			=> undef,
		precmd			=> undef,
		postcmd			=> undef,
		audiocmd		=> undef,
		audiocmd2		=> undef,
		dvdtrack		=> $dvdtrack,
		filename		=> $filename,
		width			=> undef,
		height			=> undef,
		aspect			=> undef,
		runtime			=> undef,
		dboxruntime		=> undef,
		dboxavsync		=> undef,
		fps				=> undef,
		audio_tracks	=> undef,
		adjust_width	=> undef,
		adjust_height	=> undef,
		audio_channel	=> -1,
		audio2_channel  => -1,
		probe_result	=> undef,
		preset			=> undef,
		preview_second	=> 100,
		project			=> $project,
		tc_crop_top		=> undef,
		tc_crop_bottom	=> undef,
		tc_crop_left	=> undef,
		tc_crop_right	=> undef,
		tc_scale_width	=> undef,
		tc_scale_height	=> undef,
	};
	
	return bless $self, $class;
}

sub apply_preset {
	my $self = shift;
	my %par = @_;
	my ($preset) = @par{'preset'};
	
	my $attributes = $preset->attributes;
	my $set_method;
	foreach my $attr ( @{$attributes} ) {
		$set_method = "set_$attr";
		$self->$set_method($preset->$attr);
	}
	
	1;
}

#---------------------------------------------------------------------
# Methods for taking Snapshots
#---------------------------------------------------------------------

sub snapshot_filename	   { shift->{snapshot_filename}  	 }
sub set_snapshot_filename  { shift->{snapshot_filename}  = $_[1] }

sub get_take_snapshot_command {
    my $self = shift; $self->trace_in;
	my %par = @_;
	my ($second) = @par{'second'};
	
	my $tmp_dir = "/tmp/menc$$";
	
	my $command =
	       "mkdir $tmp_dir; ".
		   "cd $tmp_dir; ".
		   $self->precmd." mplayer -nosound -vo jpeg -osdlevel 0 ".
		   "-frames 5 -ss ".($second).
		   $self->postcmd." ; ".
		   "mv 00000005.jpg snapshot.jpg; ".
		   "rm -f 0000000[0-4].jpg";
		   
	return $command;
}

sub convert_snapshot {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($filename) = @par{'filename'};

	my $tmp_dir = "/tmp/menc$$";

	my $dirname = dirname ($filename);

	my $command =
		"mkdir -p $dirname; ".
		"convert".
		" -size ".$self->width."x".$self->height.
		" $tmp_dir/snapshot.jpg $filename;".
		"rm $tmp_dir/snapshot.jpg";

	$self->system (
		command => $command
	);

	1;
}

sub take_snapshot_async_start {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my  ($second, $filename,) =
	@par{'second','filename'};
	
	$self->set_snapshot_filename ($filename);

	return $self->popen (
		command => $self->get_take_snapshot_command (
			second => $second
		),
	);
}

sub take_snapshot_async_stop {
	my $self = shift; $self->trace_in;
	my %par = @_;
	my ($fh, $output) = @par{'fh','output'};

	$output = "\n\nOutput was:\n\n$output" if $output;

	my $message =   "Error executing:\n\n".
			$self->get_take_snapshot_command (
				second => $self->preview_second
			).
			$output;

	close $fh or croak ($message);

	$self->convert_snapshot (
		filename => $self->snapshot_filename,
	);

	$self->set_snapshot_filename(undef);

	1;
}

#---------------------------------------------------------------------
# Methods for making clip and zoom images
#---------------------------------------------------------------------

sub make_preview_scale {
	my $self = shift; $self->trace_in;
	my %par = @_;

	my $source_file = $self->preview_filename( type => 'orig' );
	my $target_file = $self->preview_filename( type => 'scale' );

	my $width = $self->width;
	my $height = $self->height;
	my $aspect = $self->aspect;
	if ($height > 0 and $aspect > 0) {
		$width = $aspect * $height;
		$width = $width / 8 + 0.5;
		$width = sprintf("%d", $width);
		$width = $width * 8;
	}
				
	my $new_width  = $self->tc_scale_width;
	my $new_height = $self->tc_scale_height;
	
	if (not $new_width and $new_height) {
		$new_width = $new_height / $height * $width;
		$new_width = $new_width / 8 + 0.5;
		$new_width = sprintf("%d", $new_width);
		$new_width = $new_width * 8;		
	}
	
	if (not $new_height and $new_width) {
		$new_height = $new_width / $width * $height;
		$new_height = $new_height / 8 + 0.5;
		$new_height = sprintf("%d", $new_height);
		$new_height = $new_height * 8;	
	}	

	if ( not $new_width or not $new_height ) {
		copy ($source_file, $target_file);
		return 1;
	}
	
	my $catch = $self->system (
		command => "identify -ping $source_file"
	);
	my ($width, $height);
	($width, $height) = ( $catch =~ /(\d+)x(\d+)/ );
	
	$self->system (
		command => "convert $source_file -geometry ".
			   "'${new_width}!x${new_height}!' ".
			   $target_file
	);

	1;
}

sub make_preview_crop {
	my $self = shift; $self->trace_in;
	
	my $source_file = $self->preview_filename( type => 'scale' );

	return 1 if not -f $source_file;

	my $target_file = $self->preview_filename( type => 'crop' );
	
	my $catch = $self->system (
		command => "identify -ping $source_file"
	);
	my ($width, $height);
	($width, $height) = ( $catch =~ /(\d+)x(\d+)/ );
	
	my ($top, $bottom, $left, $right);
    $top    = $self->tc_crop_top;
	$bottom = $self->tc_crop_bottom;
	$left   = $self->tc_crop_left;
	$right  = $self->tc_crop_right;
	
	$top = 0 if not $top;
	$bottom = 0 if not $bottom;
	$left = 0 if not $left;
	$right = 0 if not $right;
	
	my $new_width  = $width - $left - $right;
	my $new_height = $height - $top - $bottom;
	
	$self->system (
		command => "convert $source_file -crop ".
			   "${new_width}x${new_height}+$left+$top ".
			   $target_file
	);

	1;
}

1;
